import pytest
from subprocess import check_call
import os
import sys


@pytest.fixture(scope="session")
def example_netstandard(tmpdir_factory):
    return build_example(tmpdir_factory, "netstandard20")


@pytest.fixture(scope="session")
def example_netcore(tmpdir_factory):
    return build_example(tmpdir_factory, "netcoreapp31")


def build_example(tmpdir_factory, framework):
    out = str(tmpdir_factory.mktemp(f"example-{framework}"))
    proj_path = os.path.join(os.path.dirname(__file__), "../example")

    check_call(["dotnet", "build", proj_path, "-o", out, "-f", framework])

    return out


def test_mono(example_netstandard):
    from clr_loader import get_mono

    mono = get_mono()
    asm = mono.get_assembly(os.path.join(example_netstandard, "example.dll"))

    run_tests(asm)


def test_coreclr(example_netcore):
    from clr_loader import get_coreclr

    coreclr = get_coreclr(os.path.join(example_netcore, "example.runtimeconfig.json"))
    asm = coreclr.get_assembly(os.path.join(example_netcore, "example.dll"))

    run_tests(asm)


@pytest.mark.skipif(
    sys.platform != "win32", reason=".NET Framework only exists on Windows"
)
def test_netfx(example_netstandard):
    from clr_loader import get_netfx

    netfx = get_netfx()
    asm = netfx.get_assembly(os.path.join(example_netstandard, "example.dll"))

    run_tests(asm)


def run_tests(asm):
    func = asm.get_function("Example.TestClass", "Test")
    test_data = b"testy mctestface"
    res = func(test_data)
    assert res == len(test_data)



# flake8: noqa

cdef = []

cdef.append(
    """
typedef struct _MonoDomain MonoDomain;
typedef struct _MonoAssembly MonoAssembly;
typedef struct _MonoImage MonoImage;
typedef struct _MonoMethodDesc MonoMethodDesc;
typedef struct _MonoMethod MonoMethod;
typedef struct _MonoObject MonoObject;

MonoDomain* mono_jit_init(const char *root_domain_name);
void mono_jit_cleanup(MonoDomain *domain);
MonoAssembly* mono_domain_assembly_open(MonoDomain *domain, const char *name);
MonoImage* mono_assembly_get_image(MonoAssembly *assembly);

void mono_config_parse(const char* path);

MonoMethodDesc* mono_method_desc_new(const char* name, bool include_namespace);
MonoMethod* mono_method_desc_search_in_image(MonoMethodDesc *method_desc, MonoImage *image);
void mono_method_desc_free(MonoMethodDesc *method_desc);

MonoObject* mono_runtime_invoke(MonoMethod *method, void *obj, void **params, MonoObject **exc);

void* mono_object_unbox(MonoObject *object);
"""
)



from .ffi import ffi, load_netfx


_FW = None


class NetFx:
    def __init__(self, name=None, config_file=None):
        global _FW
        if _FW is None:
            _FW = load_netfx()

        self._domain = _FW.pyclr_create_appdomain(
            name or ffi.NULL, config_file or ffi.NULL
        )

    def get_callable(self, assembly_path, typename, function):
        func = _FW.pyclr_get_function(
            self._domain,
            assembly_path.encode("utf8"),
            typename.encode("utf8"),
            function.encode("utf8"),
        )

        return func

    def __del__(self):
        if self._domain and _FW:
            _FW.pyclr_close_appdomain(self._domain)



#!/usr/bin/env python

from setuptools import setup, find_packages, Command, Extension
from wheel.bdist_wheel import bdist_wheel


class DotnetLib(Extension):
    def __init__(self, name, path, **kwargs):
        self.path = path
        self.args = kwargs
        super().__init__(name, sources=[])


class BuildDotnet(Command):
    """Build command for dotnet-cli based builds"""

    description = "Build DLLs with dotnet-cli"
    user_options = [("dotnet-config", None, "dotnet build configuration")]

    def initialize_options(self):
        self.dotnet_config = "release"

    def finalize_options(self):
        pass

    def get_source_files(self):
        return []

    def run(self):
        for lib in self.distribution.ext_modules:
            opts = sum(
                [
                    ["--" + name.replace("_", "-"), value]
                    for name, value in lib.args.items()
                ],
                [],
            )

            opts.append("--configuration")
            opts.append(self.dotnet_config)

            self.spawn(["dotnet", "build", lib.path] + opts)


class bdist_wheel_patched(bdist_wheel):
    def finalize_options(self):
        # Monkey patch bdist_wheel to think the package is pure even though we
        # include DLLs
        super().finalize_options()
        self.root_is_pure = True


with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="clr_loader",
    version="0.1.4",
    description="Generic pure Python loader for .NET runtimes",
    author="Benedikt Reinartz",
    author_email="filmor@gmail.com",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    python_requires=">=3.6",
    install_requires=["cffi>=1.13"],
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Operating System :: MacOS :: MacOS X",
    ],
    package_data={"clr_loader.ffi": ["dlls/x86/*.dll", "dlls/amd64/*.dll"]},
    packages=find_packages(),
    cmdclass={"build_ext": BuildDotnet, "bdist_wheel": bdist_wheel_patched},
    ext_modules={
        DotnetLib(
            "netfx-loader-x86",
            "netfx_loader/ClrLoader.csproj",
            runtime="win-x86",
            output="clr_loader/ffi/dlls/x86",
        ),
        DotnetLib(
            "netfx-loader-amd64",
            "netfx_loader/ClrLoader.csproj",
            runtime="win-x64",
            output="clr_loader/ffi/dlls/amd64",
        ),
    },
)
